'use strict';

var color = require('./color.js');
var cursor = require('./cursor.js');
var font = require('./font.js');

var global = window;
var Events = {
  BLUR: 'blur',
  CANPLAYTHROUGH: 'canplaythrough',
  CHANGE: 'change',
  CLICK: 'click',
  CONTEXTMENU: 'contextmenu',
  DBLCLICK: 'dblclick',
  DRAG: 'drag',
  DRAGEND: 'dragend',
  DRAGENTER: 'dragenter',
  DRAGLEAVE: 'dragleave',
  DRAGOVER: 'dragover',
  DRAGSTART: 'dragstart',
  DROP: 'drop',
  ENDED: 'ended',
  ERROR: 'error',
  FOCUS: 'focus',
  FOCUSIN: 'focusin',
  FOCUSOUT: 'focusout',
  GESTURECHANGE: 'gesturechange',
  GESTUREEND: 'gestureend',
  GESTURESTART: 'gesturestart',
  INPUT: 'input',
  KEYDOWN: 'keydown',
  KEYPRESS: 'keypress',
  KEYUP: 'keyup',
  LOAD: 'load',
  LOADEDMETADATA: 'loadedmetadata',
  LOSECAPTURE: 'losecapture',
  MOUSEDOWN: 'mousedown',
  MOUSEMOVE: 'mousemove',
  MOUSEOUT: 'mouseout',
  MOUSELEAVE: 'mouseleave',
  MOUSEOVER: 'mouseover',
  MOUSEUP: 'mouseup',
  MOUSEWHEEL: 'mousewheel',
  PROGRESS: 'progress',
  SCROLL: 'scroll',
  TOUCHCANCEL: 'touchcancel',
  TOUCHEND: 'touchend',
  TOUCHMOVE: 'touchmove',
  TOUCHSTART: 'touchstart'
};

function absoluteLeft(elem) {
  return elem.getBoundingClientRect().x;
}

function absoluteTop(elem) {
  return elem.getBoundingClientRect().y;
}

function on(element, eventName, handler, capturePhase) {
  element.addEventListener(eventName, handler, !!capturePhase);
  return {
    removeHandler: function removeHandler() {
      element.removeEventListener(eventName, handler, !!capturePhase);
    }
  };
}

var Orientation = {
  HORIZONTAL: 'horizontal',
  VERTICAL: 'vertical'
};
var VerticalPosition = {
  CENTER: 'center',
  TOP: 'top',
  BOTTOM: 'bottom'
};
var HorizontalPosition = {
  CENTER: 'center',
  LEFT: 'left',
  RIGHT: 'right'
};
var FontStyle = font.Style;
var ScrollBarPolicy = {
  ALLWAYS: 'allways',
  NEVER: 'never',
  AUTO: 'auto'
};

function selectFile(onSelection, fileFilter) {
  var fileField = document.createElement('input');
  fileField.type = 'file';
  if (fileFilter) fileField.accept = fileFilter;
  on(fileField, 'change', function () {
    if (fileField.files.length === 1) onSelection(fileField.files[0]);else onSelection(fileField.files);
  });
  fileField.click();
  return fileField;
}

function selectColor(onSelection, oldValue) {
  var colorField = document.createElement('input');
  colorField.type = 'color';
  if (oldValue) colorField.value = "".concat(oldValue);
  on(colorField, 'change', function () {
    onSelection(new color(colorField.value));
  });
  colorField.click();
  return colorField;
}

function later(action) {
  var timeout = setTimeout(function () {
    clearTimeout(timeout);
    action();
  }, 0);
}

function delayed(timeout, action) {
  if (arguments.length < 2) throw 'Ui.delayed needs 2 arguments (timeout, action).';
  var timeoutCookie = setTimeout(function () {
    clearTimeout(timeoutCookie);
    action();
  }, +timeout);
}

var throttle = function () {
  var watchdog = null;

  function _throttle(timeout, action) {
    if (arguments.length < 2) throw "Missing Ui.throttle 'action' argument";
    if (arguments.length < 1) throw "Missing Ui.throttle 'timeout' argument";

    function invoked() {
      watchdog = null;
      action();
    }

    if (timeout < 1) {
      // ms
      action();
    } else {
      if (!watchdog) {
        delayed(timeout, invoked);
        watchdog = invoked;
      }
    }
  }

  return _throttle;
}();

function now() {
  return new Date().valueOf();
}

var COUNTER_DIGITS = 100;
var ID = now() * COUNTER_DIGITS;
var LONG_COUNTER_DIGITS = 1000000;
var LONG_ID = now() * LONG_COUNTER_DIGITS;

function next() {
  var idMillis = Math.floor(ID / COUNTER_DIGITS);

  if (idMillis === now()) {
    var oldCounter = ID - idMillis * COUNTER_DIGITS;
    var newCounter = oldCounter + 1;

    if (newCounter === COUNTER_DIGITS) {
      // Spin with maximum duration of one millisecond ...
      var newMillis;

      do {
        newMillis = now();
      } while (newMillis === idMillis);

      ID = newMillis * COUNTER_DIGITS;
    } else {
      ID = idMillis * COUNTER_DIGITS + newCounter;
    }
  } else {
    ID = now() * COUNTER_DIGITS;
  }

  return ID;
}

function nextExtended() {
  var idMillis = Math.floor(LONG_ID / LONG_COUNTER_DIGITS);

  if (idMillis === now()) {
    var oldCounter = LONG_ID - idMillis * LONG_COUNTER_DIGITS;
    var newCounter = oldCounter + 1;

    if (newCounter === LONG_COUNTER_DIGITS) {
      // Spin with maximum duration of one millisecond ...
      var newMillis;

      do {
        newMillis = now();
      } while (newMillis === idMillis);

      LONG_ID = newMillis * LONG_COUNTER_DIGITS;
    } else {
      LONG_ID = idMillis * LONG_COUNTER_DIGITS + newCounter;
    }
  } else {
    LONG_ID = now() * LONG_COUNTER_DIGITS;
  }

  return '' + LONG_ID;
}

var module$1 = {};
Object.defineProperty(module$1, 'next', {
  enumerable: true,
  value: next
});
Object.defineProperty(module$1, 'nextExtended', {
  enumerable: true,
  value: nextExtended
});
Object.defineProperty(module$1, 'later', {
  enumerable: true,
  value: later
});
Object.defineProperty(module$1, 'delayed', {
  enumerable: true,
  value: delayed
});
Object.defineProperty(module$1, 'throttle', {
  enumerable: true,
  value: throttle
});
Object.defineProperty(module$1, 'Colors', {
  enumerable: true,
  value: color
});
Object.defineProperty(module$1, 'Color', {
  enumerable: true,
  value: color
});
Object.defineProperty(module$1, 'Cursor', {
  enumerable: true,
  value: cursor
});
Object.defineProperty(module$1, 'Font', {
  enumerable: true,
  value: font
});
Object.defineProperty(module$1, 'selectFile', {
  enumerable: true,
  value: selectFile
});
Object.defineProperty(module$1, 'selectColor', {
  enumerable: true,
  value: selectColor
});
Object.defineProperty(module$1, 'msgBox', {
  enumerable: true,
  value: global.alert
});
Object.defineProperty(module$1, 'error', {
  enumerable: true,
  value: global.alert
});
Object.defineProperty(module$1, 'warn', {
  enumerable: true,
  value: global.alert
});
Object.defineProperty(module$1, 'HorizontalPosition', {
  enumerable: true,
  value: HorizontalPosition
});
Object.defineProperty(module$1, 'VerticalPosition', {
  enumerable: true,
  value: VerticalPosition
});
Object.defineProperty(module$1, 'Orientation', {
  enumerable: true,
  value: Orientation
});
Object.defineProperty(module$1, 'ScrollBarPolicy', {
  enumerable: true,
  value: ScrollBarPolicy
});
Object.defineProperty(module$1, 'FontStyle', {
  enumerable: true,
  value: FontStyle
});
Object.defineProperty(module$1, 'on', {
  get: function get() {
    return on;
  }
});
Object.defineProperty(module$1, 'Events', {
  get: function get() {
    return Events;
  }
});
Object.defineProperty(module$1, 'absoluteLeft', {
  get: function get() {
    return absoluteLeft;
  }
});
Object.defineProperty(module$1, 'absoluteTop', {
  get: function get() {
    return absoluteTop;
  }
});

(function () {
  var popupSession = null;
  var mouseDownReg = null;

  function startPopupSession(popup) {
    function isOutsideOfAnyPopup(anElement) {
      var currentElement = anElement;

      while (currentElement !== null && currentElement !== document.body && !currentElement.classList.contains('p-menu') && !currentElement.classList.contains('p-popup')) {
        currentElement = currentElement.parentElement;
      }

      return currentElement === document.body || currentElement === null;
    }

    if (popupSession !== popup) {
      closePopupSession();
      popupSession = popup;
      mouseDownReg = on(document, Events.MOUSEDOWN, function (evt) {
        if (isOutsideOfAnyPopup(evt.target)) {
          closePopupSession();
        }
      }, true);
    }
  }

  function closePopupSession() {
    if (popupSession) {
      popupSession.close();
      popupSession = null;
    }

    if (mouseDownReg) {
      mouseDownReg.removeHandler();
      mouseDownReg = null;
    }
  }

  function isPopupSession() {
    return !!popupSession;
  }

  Object.defineProperty(module$1, 'startMenuSession', {
    get: function get() {
      return startPopupSession;
    }
  });
  Object.defineProperty(module$1, 'startPopupSession', {
    get: function get() {
      return startPopupSession;
    }
  });
  Object.defineProperty(module$1, 'closeMenuSession', {
    get: function get() {
      return closePopupSession;
    }
  });
  Object.defineProperty(module$1, 'closePopupSession', {
    get: function get() {
      return closePopupSession;
    }
  });
  Object.defineProperty(module$1, 'isMenuSession', {
    get: function get() {
      return isPopupSession;
    }
  });
  Object.defineProperty(module$1, 'isPopupSession', {
    get: function get() {
      return isPopupSession;
    }
  });
})();

module.exports = module$1;
